@@ -1,4 +1,4 @@
-# $Id: CheckOS.pm,v 1.21 2007/12/18 13:45:26 drhyde Exp $
+# $Id: CheckOS.pm,v 1.27 2008/09/26 14:28:43 drhyde Exp $
 
 package Devel::CheckOS;
 
@@ -7,7 +7,7 @@ use Exporter;
 
 use vars qw($VERSION @ISA @EXPORT_OK %EXPORT_TAGS);
 
-$VERSION = '1.43';
+$VERSION = '1.44';
 
 # localising prevents the warningness leaking out of this module
 local $^W = 1;    # use warnings is a 5.6-ism
@@ -37,6 +37,13 @@ like Linux, Solaris, AIX etc.
 
     print "You've got Linux 2.6\n" if(os_is('Linux::v2_6'));
 
+=head1 USING IT IN Makefile.PL or Build.PL
+
+If you want to use this from Makefile.PL or Build.PL, do
+not simply copy the module into your distribution as this may cause
+problems when PAUSE and search.cpan.org index the distro.  Instead, use
+the use-devel-assertos script.
+
 =head1 FUNCTIONS
 
 Devel::CheckOS implements the following functions, which load subsidiary
@@ -130,11 +137,20 @@ sub die_unsupported { die("OS unsupported\n"); }
 
 =head3 list_platforms
 
-Return a list of all the platforms for which the corresponding
+When called in list context,
+return a list of all the platforms for which the corresponding
 Devel::AssertOS::* module is available.  This includes both OSes and OS
 families, and both those bundled with this module and any third-party
 add-ons you have installed.
 
+In scalar context, returns a hashref keyed by platform with the filename
+of the most recent version of the supporting module that is available to you.
+This is to make sure that the use-devel-assertos script Does The Right Thing
+in the case where you have installed the module in one version of perl, then
+upgraded perl, and installed it again in the new version.  Sometimes the old
+version of perl and all its modules will still be hanging around and perl
+"helpfully" includes the old perl's search path in its own.
+
 Unfortunately, on some platforms this list may have file case
 broken.  eg, some platforms might return 'freebsd' instead of 'FreeBSD'.
 This is because they have case-insensitive filesystems so things
@@ -156,7 +172,11 @@ sub list_platforms {
         $re_Devel    = qr/$case_flag ^Devel$/x;
         $re_AssertOS = qr/$case_flag ^AssertOS$/x;
     }
-    return sort { $a cmp $b } map {
+
+    # sort by mtime, so oldest last
+    my @modules = sort {
+        (stat($a->{file}))[9] <=> (stat($b->{file}))[9]
+    } map {
         my (undef, $dir_part, $file_part) = File::Spec->splitpath($_);
         $file_part =~ s/\.pm$//;
         my (@dirs) = grep {+length} File::Spec->splitdir($dir_part);
@@ -166,12 +186,25 @@ sub list_platforms {
             splice @dirs, 0, $i + 1;
             last;
         }
-        join '::', @dirs, $file_part
+        {
+	    module => join('::', @dirs, $file_part),
+	    file   => $_
+	}
     } File::Find::Rule->file()->name('*.pm')->in(
         grep { -d }
         map { File::Spec->catdir($_, qw(Devel AssertOS)) }
         @INC
     );
+
+    my %modules = map {
+        $_->{module} => $_->{file}
+    } @modules;
+
+    if(wantarray()) {
+        return sort keys %modules;
+    } else {
+        return \%modules;
+    }
 }
 
 =head1 PLATFORMS SUPPORTED
@@ -242,6 +275,10 @@ information about what should be in the Unix family.
 
 Thanks to Billy Abbott for finding some bugs for me on VMS.
 
+=head1 CVS
+
+L<http://drhyde.cvs.sourceforge.net/drhyde/perlmodules/Devel-CheckOS/>
+
 =head1 COPYRIGHT and LICENCE
 
 Copyright 2007 David Cantrell
